﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Drawing;
using MindFusion.Diagramming;
using MindFusion.Diagramming.Mvc;
using MindFusion.Diagramming.Layout;
using LinearGradientBrush = MindFusion.Drawing.LinearGradientBrush;

namespace LayeredLayout.Controllers
{
    public class HomeController : Controller
    {
        public ActionResult Index(string submitButton)
        {
            DiagramView view = DiagramView.FromRequest("diagramView1", Request);
            view.SetClientSideMode(ClientSideMode.MsAjax);
            ViewBag.DiagramView = view;

            Overview overview = new Overview("overview1", "diagramView1");
            overview.DimColor = System.Drawing.Color.White;
            ViewBag.Overview = overview;

            switch (submitButton)
            {
                case "Random graph":
                    return Init(view);
                case "Zoom in":
                    {
                        if (view.ZoomFactor < 200)
                            view.ZoomFactor += 10;
                        break;
                    }
                case "Zoom out":
                    {
                        if (view.ZoomFactor > 19)
                            view.ZoomFactor -= 10;
                        break;
                    }
            }
            return View();
        }

        public ActionResult Init(DiagramView view)
        {
            Diagram diagram = view.Diagram;
            BuildTree(diagram);

            var layout = new MindFusion.Diagramming.Layout.LayeredLayout();
            layout.NodeDistance = 15;
            layout.LayerDistance = 15;
            layout.Arrange(diagram);
            diagram.ResizeToFitItems(5);

            ViewBag.DiagramView = view;
            return View();
        }

        private void BuildTree(Diagram diagram)
        {
            var r = new Random();
            int n = 10;
            diagram.ClearAll();

            for (var i = 0; i < n; ++i)
            {
                var c = diagram.Nodes.Count;
                var g = 2 + r.Next(15);
                for (var j = 0; j < g; ++j)
                {
                    var rect = new RectangleF(0, 0, 10, 10);
                    var node = new ShapeNode(diagram);
                    node.Bounds = rect;
                    node.Brush = new LinearGradientBrush(Color.White, ColorTranslator.FromHtml("#ce0000"), 90);
                    diagram.Nodes.Add(node);
                    if (j > 0)
                    {
                        var link = new DiagramLink(
                            diagram, diagram.Nodes[diagram.Nodes.Count - 2], node);
                        diagram.Links.Add(link);
                    }
                }
                if (i > 0)
                {
                    for (var j = 0; j < 1 + r.Next(3); ++j)
                    {
                        var link = new DiagramLink(
                            diagram, diagram.Nodes[r.Next(c)], diagram.Nodes[c + r.Next(g)]);
                        diagram.Links.Add(link);
                    }
                }
            }
        }
    }
}